@@ -7,7 +7,7 @@ Attean::API - Utility package for loading all Attean role packages.
 
 =head1 VERSION
 
-This document describes Attean::API version 0.005
+This document describes Attean::API version 0.006
 
 =head1 SYNOPSIS
 
@@ -21,28 +21,50 @@ in the Attean::API namespace.
 
 =cut
 
-package Attean::API::ResultOrTerm 0.005 {
+package Attean::API::ResultOrTerm 0.006 {
 	use Moo::Role;
 }
 
-package Attean::API::BlankOrIRI 0.005 {
+package Attean::API::BlankOrIRI 0.006 {
 	use Moo::Role;
 }
 
-package Attean::API::TermOrVariable 0.005 {
+package Attean::API::TermOrVariable 0.006 {
 	use Moo::Role;
+	use Sub::Install;
+	use Sub::Util qw(set_subname);
+	use namespace::clean;
+
 	sub equals {
 		my ($a, $b)	= @_;
 		return ($a->as_string eq $b->as_string);
 	}
+	
+	BEGIN {
+		my %types	= (
+			variable	=> 'Variable',
+			blank		=> 'Blank',
+			literal		=> 'Literal',
+			resource	=> 'IRI',
+			iri			=> 'IRI',
+		);
+		while (my ($name, $role) = each(%types)) {
+			my $method	= "is_$name";
+			my $code	= sub { return shift->does("Attean::API::$role") };
+			Sub::Install::install_sub({
+				code	=> set_subname($method, $code),
+				as		=> $method
+			});
+		}
+	}
 }
 
-package Attean::Mapper 0.005 {
+package Attean::Mapper 0.006 {
 	use Moo::Role;
 	requires 'map'; # my $that = $object->map($this)
 }
 
-package Attean::API::Variable 0.005 {
+package Attean::API::Variable 0.006 {
 	use Moo::Role;
 	with 'Attean::API::TermOrVariable';
 	sub as_sparql {
@@ -56,7 +78,7 @@ package Attean::API::Variable 0.005 {
 	}
 }
 
-package Attean::API::CanonicalizingBindingSet 0.005 {
+package Attean::API::CanonicalizingBindingSet 0.006 {
 	use Moo::Role;
 	use Attean::RDF;
 	use namespace::clean;
@@ -147,7 +169,7 @@ package Attean::API::CanonicalizingBindingSet 0.005 {
 	}
 }
 
-package Attean::API 0.005 {
+package Attean::API 0.006 {
 	use Attean::API::Term;
 	use Attean::API::Store;
 	use Attean::API::Model;
