@@ -7,7 +7,7 @@ Attean::API::Query - Utility package defining query-related roles
 
 =head1 VERSION
 
-This document describes Attean::API::Query version 0.005
+This document describes Attean::API::Query version 0.006
 
 =head1 SYNOPSIS
 
@@ -24,7 +24,7 @@ This is a utility package for defining query-related roles:
 
 =cut
 
-package Attean::API::DirectedAcyclicGraph 0.005 {
+package Attean::API::DirectedAcyclicGraph 0.006 {
 	use Moo::Role;
 	use Scalar::Util qw(refaddr);
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -80,6 +80,33 @@ package Attean::API::DirectedAcyclicGraph 0.005 {
 		}
 	}
 	
+# =item C<< has_only_subtree_types( @classes ) >>
+# 
+# Returns true if the invocant and all of its sub-trees are instances of only
+# the listed classes, false otherwise.
+# 
+# =cut
+
+	sub has_only_subtree_types {
+		my $self	= shift;
+		my @types	= @_;
+		my %types	= map { $_ => 1 } @types;
+		return 0 unless (exists $types{ ref($self) });
+		
+		my %classes;
+		$self->walk( prefix => sub {
+			my $plan	= shift;
+			$classes{ref($plan)}++;
+		});
+		foreach my $type (@types) {
+			delete $classes{$type};
+		}
+		use Data::Dumper;
+		warn Dumper(\%classes);
+		my @keys	= keys %classes;
+		return (scalar(@keys) == 0) ? 1 : 0;
+	}
+
 # =item C<< cover( prefix => \&pre_cb, postfix => \&pre_cb ) >>
 # 
 # Similar to C<< walk >>, walks the graph rooted at the referent, calling
@@ -115,7 +142,7 @@ package Attean::API::DirectedAcyclicGraph 0.005 {
 
 =cut
 
-package Attean::API::Algebra 0.005 {
+package Attean::API::Algebra 0.006 {
 	use Moo::Role;
 
 	requires 'as_sparql';
@@ -164,7 +191,7 @@ package Attean::API::Algebra 0.005 {
 
 =cut
 
-package Attean::API::QueryTree 0.005 {
+package Attean::API::QueryTree 0.006 {
 	use Moo::Role;
 	with 'Attean::API::DirectedAcyclicGraph';
 }
@@ -173,7 +200,7 @@ package Attean::API::QueryTree 0.005 {
 
 =cut
 
-package Attean::API::NullaryQueryTree 0.005 {
+package Attean::API::NullaryQueryTree 0.006 {
 	use Moo::Role;
 	sub arity { return 0 }
 	with 'Attean::API::QueryTree';
@@ -183,7 +210,7 @@ package Attean::API::NullaryQueryTree 0.005 {
 
 =cut
 
-package Attean::API::UnaryQueryTree 0.005 {
+package Attean::API::UnaryQueryTree 0.006 {
 	use Moo::Role;
 	sub arity { return 1 }
 	with 'Attean::API::QueryTree';
@@ -193,7 +220,7 @@ package Attean::API::UnaryQueryTree 0.005 {
 
 =cut
 
-package Attean::API::BinaryQueryTree 0.005 {
+package Attean::API::BinaryQueryTree 0.006 {
 	use Moo::Role;
 	sub arity { return 2 }
 	with 'Attean::API::QueryTree';
@@ -203,7 +230,7 @@ package Attean::API::BinaryQueryTree 0.005 {
 
 =cut
 
-package Attean::API::PropertyPath 0.005 {
+package Attean::API::PropertyPath 0.006 {
 	use Moo::Role;
 	with 'Attean::API::QueryTree';
 	requires 'as_string';
@@ -214,7 +241,7 @@ package Attean::API::PropertyPath 0.005 {
 
 =cut
 
-package Attean::API::UnaryPropertyPath 0.005 {
+package Attean::API::UnaryPropertyPath 0.006 {
 	use Moo::Role;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
@@ -244,7 +271,7 @@ package Attean::API::UnaryPropertyPath 0.005 {
 
 =cut
 
-package Attean::API::NaryPropertyPath 0.005 {
+package Attean::API::NaryPropertyPath 0.006 {
 	use Moo::Role;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -271,7 +298,7 @@ package Attean::API::NaryPropertyPath 0.005 {
 
 =cut
 
-package Attean::API::UnionScopeVariables 0.005 {
+package Attean::API::UnionScopeVariables 0.006 {
 	use Moo::Role;
 	sub in_scope_variables {
 		my $self	= shift;
@@ -287,7 +314,7 @@ package Attean::API::UnionScopeVariables 0.005 {
 
 =cut
 
-package Attean::API::IntersectionScopeVariables 0.005 {
+package Attean::API::IntersectionScopeVariables 0.006 {
 	use Moo::Role;
 	sub in_scope_variables {
 		my $self	= shift;
