@@ -7,7 +7,7 @@ Attean::IDPQueryPlanner - Iterative dynamic programming query planner
 
 =head1 VERSION
 
-This document describes Attean::IDPQueryPlanner version 0.005
+This document describes Attean::IDPQueryPlanner version 0.006
 
 =head1 SYNOPSIS
 
@@ -34,7 +34,7 @@ use Attean::Algebra;
 use Attean::Plan;
 use Attean::Expression;
 
-package Attean::IDPQueryPlanner 0.005 {
+package Attean::IDPQueryPlanner 0.006 {
 	use Moo;
 	use Encode qw(encode);
 	use Attean::RDF;
@@ -97,9 +97,33 @@ the supplied C<< $active_graph >>.
 		my @children	= @{ $algebra->children };
 		my ($child)		= $children[0];
 		if ($algebra->isa('Attean::Algebra::BGP')) {
-			my @plans	= $self->bgp_join_plans($algebra, $model, $active_graphs, $default_graphs, $interesting, map {
+			my $triples	= $algebra->triples;
+			my @triples	= @$triples;
+			my %blanks;
+			foreach my $i (0 .. $#triples) {
+				my $t	= $triples[$i];
+				my @nodes	= $t->values;
+				my $changed	= 0;
+				foreach (@nodes) {
+					if ($_->does('Attean::API::Blank')) {
+						$changed++;
+						my $id	= $_->value;
+						unless (exists $blanks{$id}) {
+							$blanks{$id}	= variable($self->new_temporary('blank'));
+						}
+						$_	= $blanks{$id};
+					}
+				}
+				
+				if ($changed) {
+					my $new	= triple(@nodes);
+					$triples[$i]	= $new;
+				}
+			}
+			my $bgp		= Attean::Algebra::BGP->new( triples => \@triples );
+			my @plans	= $self->bgp_join_plans($bgp, $model, $active_graphs, $default_graphs, $interesting, map {
 				[$self->access_plans($model, $active_graphs, $_)]
-			} @{ $algebra->triples });
+			} @triples);
 			return @plans;
 		} elsif ($algebra->isa('Attean::Algebra::Join')) {
 			return $self->group_join_plans($model, $active_graphs, $default_graphs, $interesting, map {
@@ -177,9 +201,11 @@ the supplied C<< $active_graph >>.
 		} elsif ($algebra->isa('Attean::Algebra::LeftJoin')) {
 			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, @_)];
 			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, @_)];
-			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 1, 0, $algebra->expression);
+			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 'left', $algebra->expression);
 		} elsif ($algebra->isa('Attean::Algebra::Minus')) {
-			return $self->join_plans($model, $active_graphs, $default_graphs, @children[0,1], 0, 1);
+			my $l	= [$self->plans_for_algebra($children[0], $model, $active_graphs, $default_graphs, @_)];
+			my $r	= [$self->plans_for_algebra($children[1], $model, $active_graphs, $default_graphs, @_)];
+			return $self->join_plans($model, $active_graphs, $default_graphs, $l, $r, 'minus');
 		} elsif ($algebra->isa('Attean::Algebra::Project')) {
 			my $vars	= $algebra->variables;
 			my @vars	= map { $_->value } @{ $vars };
@@ -338,6 +364,14 @@ triple participating in the join.
 					$distinct	= 0;
 					last LOOP;
 				}
+				foreach my $b ($t->values_consuming_role('Attean::API::Variable')) {
+					if ($b->value =~ /^[.]/) {
+						# variable names starting with a dot represent placeholders introduced during query planning (with C<new_temporary>)
+						# they are not projectable, and so may cause an otherwise distinct result to become non-distinct
+						$distinct	= 0;
+						last LOOP;
+					}
+				}
 			}
 		
 			# Set the distinct flag on each of the top-level join plans that
@@ -409,7 +443,7 @@ sub-plan participating in the join.
 						my $rhs		= $optPlan{$not_o_key}; # get the plans for evaluating not_o
 						
 						# compute and store all the possible ways to evaluate s (o ⋈ not_o)
-						push(@{ $optPlan{$s_key} }, $self->join_plans($model, $active_graphs, $default_graphs, $lhs, $rhs, 0, 0));
+						push(@{ $optPlan{$s_key} }, $self->join_plans($model, $active_graphs, $default_graphs, $lhs, $rhs, 'inner'));
 						$optPlan{$s_key}	= [$self->prune_plans($model, $interesting, $optPlan{$s_key})];
 					}
 				}
@@ -475,7 +509,7 @@ sub-plan participating in the join.
 		my $model		= shift;
 		my $interesting	= shift;
 		my @plans		= @{ shift || [] };
-		
+	        no  sort 'stable';
 		my @sorted	= map { $_->[1] } sort { $a->[0] <=> $b->[0] } map { [$self->cost_for_plan($_, $model), $_] } @plans;
 		return splice(@sorted, 0, 5);
 	}
@@ -488,7 +522,7 @@ sub-plan participating in the join.
 		my $model			= shift;
 		my $active_graphs	= shift;
 		my $pattern			= shift;
-		my @vars			= map { $_->value } grep { $_->does('Attean::API::Variable') } $pattern->values;
+		my @vars			= map { $_->value } $pattern->values_consuming_role('Attean::API::Variable');
 		my %vars;
 		my $dup				= 0;
 		foreach my $v (@vars) {
@@ -525,8 +559,9 @@ sub-plan participating in the join.
 		my $default_graphs	= shift;
 		my $lplans			= shift;
 		my $rplans			= shift;
-		my $left			= shift;
-		my $minus			= shift;
+		my $type			= shift;
+		my $left			= ($type eq 'left');
+		my $minus			= ($type eq 'minus');
 		my $expr			= shift;
 		
 		my @plans;
@@ -535,19 +570,14 @@ sub-plan participating in the join.
 			foreach my $rhs (@{ $rplans }) {
 				my @vars	= (@{ $lhs->in_scope_variables }, @{ $rhs->in_scope_variables });
 				my %vars;
-				foreach my $v (@vars) {
-					$vars{$v}++;
-				}
 				my %join_vars;
-				foreach my $l (@{ $lhs->in_scope_variables }) {
-					foreach my $r (@{ $rhs->in_scope_variables }) {
-						if ($l eq $r) {
-							$join_vars{$l}++;
-						}
+				foreach my $v (@vars) {
+					if ($vars{$v}++) {
+						$join_vars{$v}++;
 					}
 				}
 				my @join_vars	= keys %join_vars;
-		
+				
 				if ($left) {
 					push(@plans, Attean::Plan::NestedLoopJoin->new(children => [$lhs, $rhs], left => 1, expression => $expr, join_variables => \@join_vars, distinct => 0, in_scope_variables => [keys %vars], ordered => $lhs->ordered));
 					if (scalar(@join_vars) > 0) {
@@ -598,10 +628,9 @@ sub-plan participating in the join.
 			my @children	= @{ $plan->children };
 			if ($plan->isa('Attean::Plan::Quad')) {
 				my @vars	= map { $_->value } grep { blessed($_) and $_->does('Attean::API::Variable') } $plan->values;
-				return 3 * scalar(@vars);
+				# This gives a cost increasing at a reasonable pace
+				return $self->_hsp_heuristic_triple_sum($plan) * scalar(@vars);
 			} elsif ($plan->isa('Attean::Plan::NestedLoopJoin')) {
-				my $jv			= $plan->join_variables;
-				my $mult		= scalar(@$jv) ? 1 : 5;	# penalize cartesian joins
 				my $lcost		= $self->cost_for_plan($children[0], $model);
 				my $rcost		= $self->cost_for_plan($children[1], $model);
 				if ($lcost == 0) {
@@ -609,11 +638,10 @@ sub-plan participating in the join.
 				} elsif ($rcost == 0) {
 					$cost	= $lcost;
 				} else {
+					my $mult = $self->_penalize_joins($plan);
 					$cost	= $mult * $lcost * $rcost;
 				}
 			} elsif ($plan->isa('Attean::Plan::HashJoin')) {
-				my $jv			= $plan->join_variables;
-				my $mult		= scalar(@$jv) ? 1 : 5;	# penalize cartesian joins
 				my $lcost		= $self->cost_for_plan($children[0], $model);
 				my $rcost		= $self->cost_for_plan($children[1], $model);
 				if ($lcost == 0) {
@@ -621,6 +649,7 @@ sub-plan participating in the join.
 				} elsif ($rcost == 0) {
 					$cost	= $lcost;
 				} else {
+					my $mult = $self->_penalize_joins($plan);
 					$cost	= $mult * ($lcost + $rcost);
 				}
 			} elsif ($plan->isa('Attean::Plan::Unique')) {
@@ -665,6 +694,111 @@ sub-plan participating in the join.
 		my @keys	= keys %unseen;
 		return (scalar(@keys) == 0);
 	}
+
+# The below function finds a number to aid sorting
+# It takes into account Heuristic 1 and 4 of the HSP paper, see REFERENCES
+# as well as that it was noted in the text that rdf:type is usually less selective.
+
+# By assigning the integers to nodes, depending on whether they are in
+# triple (subject, predicate, object), variables, rdf:type and
+# literals, and sum them, they may be sorted. See code for the actual
+# values used.
+
+# Denoting s for bound subject, p for bound predicate, a for rdf:type
+# as predicate, o for bound object and l for literal object and ? for
+# variable, we get the following order, most of which are identical to
+# the HSP:
+
+# spl: 6
+# spo: 8
+# sao: 10
+# s?l: 14
+# s?o: 16
+# ?pl: 25
+# ?po: 27
+# ?ao: 29
+# sp?: 30
+# sa?: 32
+# ??l: 33
+# ??o: 35
+# s??: 38
+# ?p?: 49
+# ?a?: 51
+# ???: 57
+
+# Note that this number is not intended as an estimate of selectivity,
+# merely a sorting key, but further research may possibly create such
+# numbers.
+
+	sub _hsp_heuristic_triple_sum {
+		my ($self, $t) = @_;
+		my $sum = 0;
+		if ($t->subject->does('Attean::API::Variable')) {
+			$sum = 20;
+		} else {
+			$sum = 1;
+		}
+		if ($t->predicate->does('Attean::API::Variable')) {
+			$sum += 10;
+		} else {
+			if ($t->predicate->equals(iri('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'))) {
+				$sum += 4;
+			} else {
+				$sum += 2;
+			}
+		}
+		if ($t->object->does('Attean::API::Variable')) {
+			$sum += 27;
+		} elsif ($t->object->does('Attean::API::Literal')) {
+			$sum += 3;
+		} else {
+			$sum += 5;
+		}
+		return $sum;
+	}
+
+	# The following method returns a factor used to penalize certain types of joins.
+	# It penalizes cartesian joins heavily, but also uses HSP Heuristic 2 (see REFERENCES)
+	sub _penalize_joins {
+		my ($self, $plan) = @_;
+		my $jv			= $plan->join_variables;
+		my @children	= @{ $plan->children };
+		my $mult		   = 1;
+		if (scalar(@$jv)) {
+			if ( all { $_->isa('Attean::Plan::Quad') } @children[0..1]) {
+				my $var = ${$jv}[0]; # We will join on this
+				my @lnodes = $children[0]->values;
+				my @rnodes = $children[1]->values;
+				# Now, find where the join variables are in the triple patterns
+				my %joinpos;
+				for (my $i = 0; $i <= 2; $i++) {
+					if ($lnodes[$i]->does('Attean::API::Variable') && $lnodes[$i]->value eq $var) {
+						$joinpos{l} = $i;
+					}
+					if ($rnodes[$i]->does('Attean::API::Variable') && $rnodes[$i]->value eq $var) {
+						$joinpos{r} = $i;
+					}
+					last if scalar keys(%joinpos) >= 2; # Perhaps a bit premature optimization
+				}
+				my $joinpos = join("", sort values(%joinpos)); # We can now match on this string
+				my %costs = ('12' => 1.1, # The penalty numbers come mostly out from thin air
+								 '01' => 1.2,
+								 '02' => 1.5,
+								 '22' => 1.6,
+								 '00' => 1.8,
+								 '11' => 2);
+				if (exists $costs{$joinpos}) {
+					$mult = $costs{$joinpos};
+				}
+				#warn "Penalty: $mult for quads:\n" . $children[0]->as_string . $children[1]->as_string
+			}
+		} else {
+			$mult = 5; # penalize cartesian joins
+		}
+		return $mult;
+	}
+
+
 }
 
 1;
@@ -678,6 +812,18 @@ __END__
 Please report any bugs or feature requests to through the GitHub web interface
 at L<https://github.com/kasei/attean/issues>.
 
+=head1 REFERENCES
+
+The seminal reference for Iterative Dynamic Programming is "Iterative
+dynamic programming: a new class of query optimization algorithms" by
+D. Kossmann and K. Stocker, ACM Transactions on Database Systems
+(2000).
+
+The heuristics to order triple patterns in this module is
+influenced by L<The ICS-FORTH Heuristics-based SPARQL Planner
+(HSP)|http://www.ics.forth.gr/isl/index_main.php?l=e&c=645>.
+
+
 =head1 SEE ALSO
 
 L<http://www.perlrdf.org/>
