@@ -7,7 +7,7 @@ Attean::Plan - Representation of SPARQL query plan operators
 
 =head1 VERSION
 
-This document describes Attean::Plan version 0.004
+This document describes Attean::Plan version 0.006
 
 =head1 SYNOPSIS
 
@@ -27,16 +27,25 @@ use Attean::API::Query;
 
 =item * L<Attean::Plan::Quad>
 
+Evaluates a quad pattern against the model.
+
 =cut
 
-package Attean::Plan::Quad 0.004 {
+package Attean::Plan::Quad 0.006 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf ArrayRef);
+
+	has 'subject'	=> (is => 'ro', required => 1);
+	has 'predicate'	=> (is => 'ro', required => 1);
+	has 'object'	=> (is => 'ro', required => 1);
+	has 'graph'		=> (is => 'ro', required => 1);
+	
 	with 'Attean::API::Plan', 'Attean::API::NullaryQueryTree';
-	has 'values' => (is => 'ro', isa => ArrayRef, default => sub { [] });
+	with 'Attean::API::QuadPattern';
+
 	sub plan_as_string {
 		my $self	= shift;
-		my @nodes	= @{ $self->values };
+		my @nodes	= $self->values;
 		my @strings;
 		foreach my $t (@nodes) {
 			if (ref($t) eq 'ARRAY') {
@@ -59,7 +68,7 @@ package Attean::Plan::Quad 0.004 {
 	sub impl {
 		my $self	= shift;
 		my $model	= shift;
-		my @values	= @{ $self->values };
+		my @values	= $self->values;
 		return sub {
 			return $model->get_bindings( @values );
 		}
@@ -68,9 +77,11 @@ package Attean::Plan::Quad 0.004 {
 
 =item * L<Attean::Plan::NestedLoopJoin>
 
+Evaluates a join (natural-, anti-, or left-) using a nested loop.
+
 =cut
 
-package Attean::Plan::NestedLoopJoin 0.004 {
+package Attean::Plan::NestedLoopJoin 0.006 {
 	use Moo;
 	use Types::Standard qw(ArrayRef Str Bool ConsumerOf);
 	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
@@ -129,9 +140,11 @@ package Attean::Plan::NestedLoopJoin 0.004 {
 
 =item * L<Attean::Plan::HashJoin>
 
+Evaluates a join (natural-, anti-, or left-) using a hash join.
+
 =cut
 
-package Attean::Plan::HashJoin 0.004 {
+package Attean::Plan::HashJoin 0.006 {
 	use Moo;
 	use Types::Standard qw(ArrayRef Str ConsumerOf Bool);
 	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
@@ -200,33 +213,76 @@ package Attean::Plan::HashJoin 0.004 {
 	}
 }
 
-=item * L<Attean::Plan::Filter>
+=item * L<Attean::Plan::EBVFilter>
+
+Filters results from a sub-plan based on the effective boolean value of a
+named variable binding.
 
 =cut
 
-package Attean::Plan::Filter 0.004 {
+package Attean::Plan::EBVFilter 0.006 {
 	use Moo;
-	use Types::Standard qw(ConsumerOf);
+	use Scalar::Util qw(blessed);
+	use Types::Standard qw(Str ConsumerOf);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
-	has 'expression' => (is => 'ro', isa => ConsumerOf['Attean::API::Expression'], required => 1);
+	has 'variable' => (is => 'ro', isa => Str, required => 1);
 	sub plan_as_string {
 		my $self	= shift;
-		return sprintf('Filter { %s }', $self->expression->as_string);
+		return sprintf('EBVFilter { ?%s }', $self->variable);
 	}
 	sub tree_attributes { return qw(expression) };
 	
 	sub impl {
-		# TODO: implement
-		die "Unimplemented";
+		my $self	= shift;
+		my $model	= shift;
+		my ($impl)	= map { $_->impl($model) } @{ $self->children };
+		my $var		= $self->variable;
+		return sub {
+			my $iter	= $impl->();
+			return $iter->grep(sub {
+				my $r		= shift;
+				my $term	= $r->value($var);
+				return 0 unless (blessed($term) and $term->does('Attean::API::Term'));
+				return $term->ebv;
+			});
+		};
+	}
+}
+
+=item * L<Attean::Plan::Merge>
+
+Evaluates a set of sub-plans, returning the merged union of results, preserving
+ordering.
+
+=cut
+
+package Attean::Plan::Merge 0.006 {
+	use Moo;
+	use Scalar::Util qw(blessed);
+	use Types::Standard qw(Str ArrayRef ConsumerOf);
+	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
+	has 'variables' => (is => 'ro', isa => ArrayRef[Str], required => 1);
+	sub plan_as_string { return 'Merge' }
+	
+	sub impl {
+		my $self	= shift;
+		my $model	= shift;
+		my @children	= map { $_->impl($model) } @{ $self->children };
+		return sub {
+			die "Unimplemented";
+		};
 	}
 }
 
 =item * L<Attean::Plan::Union>
 
+Evaluates a set of sub-plans, returning the union of results.
+
 =cut
 
-package Attean::Plan::Union 0.004 {
+package Attean::Plan::Union 0.006 {
 	use Moo;
+	use Scalar::Util qw(blessed);
 	with 'Attean::API::Plan', 'Attean::API::BinaryQueryTree';
 	sub plan_as_string { return 'Union' }
 	
@@ -235,17 +291,25 @@ package Attean::Plan::Union 0.004 {
 		my $model	= shift;
 		my @children	= map { $_->impl($model) } @{ $self->children };
 		return sub {
-			my @results;
-			while (my $current = shift(@children)) {
-				my $iter	= $current->();
-				while (my $row = $iter->next()) {
-					push(@results, $row);
-				}
-			}
-			
-			return Attean::ListIterator->new(
+			my $current	= shift(@children);
+			my $iter	= $current->();
+			return Attean::CodeIterator->new(
 				item_type => 'Attean::API::Result',
-				values => \@results
+				generator => sub {
+					while (blessed($iter)) {
+						my $row	= $iter->next();
+						if ($row) {
+							return $row;
+						} else {
+							$current	= shift(@children);
+							if ($current) {
+								$iter	= $current->();
+							} else {
+								undef $iter;
+							}
+						}
+					}
+				},
 			);
 		};
 	}
@@ -253,13 +317,17 @@ package Attean::Plan::Union 0.004 {
 
 =item * L<Attean::Plan::Extend>
 
+Evaluates a sub-plan, and extends each result by evaluating a set of
+expressions, binding the produced values to new variables.
+
 =cut
 
-package Attean::Plan::Extend 0.004 {
+package Attean::Plan::Extend 0.006 {
 	use Moo;
+	use Scalar::Util qw(blessed);
 	use Types::Standard qw(ConsumerOf HashRef);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
-	has 'expressions' => (is => 'ro', isa => HashRef[ConsumerOf['Attean::API::Expression']], ,required => 1);
+	has 'expressions' => (is => 'ro', isa => HashRef[ConsumerOf['Attean::API::Expression']], required => 1);
 	sub plan_as_string {
 		my $self	= shift;
 		my @strings	= map { sprintf('?%s ← %s', $_, $self->expressions->{$_}->as_string) } keys %{ $self->expressions };
@@ -267,14 +335,18 @@ package Attean::Plan::Extend 0.004 {
 	}
 	sub tree_attributes { return qw(variable expression) };
 	
-	sub evaluate {
+	sub evaluate_expression {
 		my $self	= shift;
 		my $model	= shift;
 		my $expr	= shift;
 		my $r		= shift;
 		my $op		= $expr->operator;
-		my $true	= Attean::Literal->true;
-		my $false	= Attean::Literal->false;
+
+		state $true			= Attean::Literal->true;
+		state $false		= Attean::Literal->false;
+		state $type_roles	= { qw(URI IRI IRI IRI BLANK Blank LITERAL Literal NUMERIC NumericLiteral) };
+		state $type_classes	= { qw(URI Attean::IRI IRI Attean::IRI STR Attean::Literal) };
+		
 		if ($expr->isa('Attean::ValueExpression')) {
 			my $node	= $expr->value;
 			if ($node->does('Attean::API::Variable')) {
@@ -284,7 +356,7 @@ package Attean::Plan::Extend 0.004 {
 			}
 		} elsif ($expr->isa('Attean::UnaryExpression')) {
 			my ($child)	= @{ $expr->children };
-			my $term	= $self->evaluate($model, $child, $r);
+			my $term	= $self->evaluate_expression($model, $child, $r);
 			if ($op eq '!') {
 				return ($term->ebv) ? $false : $true;
 			} elsif ($op eq '-' or $op eq '+') {
@@ -293,6 +365,17 @@ package Attean::Plan::Extend 0.004 {
 				return Attean::Literal->new( value => eval "$op$v", datatype => $term->datatype );
 			}
 			die "Unimplemented UnaryExpression evaluation: " . $expr->operator;
+		} elsif ($expr->isa('Attean::FunctionExpression')) {
+			my $func	= $expr->operator;
+			my @terms	= map { $self->evaluate_expression($model, $_, $r) } @{ $expr->children };
+			if ($func =~ /^IS([UI]RI|BLANK|LITERAL|NUMERIC)$/) {
+				my $role	= "Attean::API::$type_roles->{$1}";
+				my $t		= shift(@terms);
+				my $ok		= (blessed($t) and $t->does($role));
+				return $ok ? $true : $false;
+			} else {
+				die "Expression evaluation unimplemented: " . $expr->as_string;
+			}
 		} else {
 			die "Expression evaluation unimplemented: " . $expr->as_string;
 		}
@@ -305,40 +388,42 @@ package Attean::Plan::Extend 0.004 {
 		my ($impl)	= map { $_->impl($model) } @{ $self->children };
 		return sub {
 			my $iter	= $impl->();
-			my @results;
-			ROW: while (my $r = $iter->next) {
-				my %row	= map { $_ => $r->value($_) } $r->variables;
-				foreach my $var (keys %exprs) {
-					my $expr	= $exprs{$var};
-					my $term	= $self->evaluate($model, $expr, $r);
-					if ($term and $row{ $var } and $term->as_string ne $row{ $var }->as_string) {
-						next ROW;
-					}
-					
-					if ($term) {
-						$row{ $var }	= $term;
-					}
-				}
-				push(@results, Attean::Result->new( bindings => \%row ));
-			}
-			return Attean::ListIterator->new(
+			return Attean::CodeIterator->new(
 				item_type => 'Attean::API::Result',
-				values => \@results
+				generator => sub {
+					ROW: while (my $r = $iter->next) {
+						my %row	= map { $_ => $r->value($_) } $r->variables;
+						foreach my $var (keys %exprs) {
+							my $expr	= $exprs{$var};
+							my $term	= $self->evaluate_expression($model, $expr, $r);
+							if ($term and $row{ $var } and $term->as_string ne $row{ $var }->as_string) {
+								next ROW;
+							}
+					
+							if ($term) {
+								$row{ $var }	= $term;
+							}
+						}
+						return Attean::Result->new( bindings => \%row );
+					}
+					return;
+				}
 			);
 		};
-		# TODO: implement
-		die "Unimplemented";
 	}
 }
 
-=item * L<Attean::Plan::Distinct>
+=item * L<Attean::Plan::HashDistinct>
+
+Evaluates a sub-plan, and returns distinct results by checking a persistent
+hash of already-seen results.
 
 =cut
 
-package Attean::Plan::Distinct 0.004 {
+package Attean::Plan::HashDistinct 0.006 {
 	use Moo;
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
-	sub plan_as_string { return 'Distinct' }
+	sub plan_as_string { return 'HashDistinct' }
 	
 	sub impl {
 		my $self	= shift;
@@ -352,11 +437,45 @@ package Attean::Plan::Distinct 0.004 {
 	}
 }
 
-=item * L<Attean::Plan::Slice>
+=item * L<Attean::Plan::Unique>
+
+Evaluates an already-ordered sub-plan, and returns distinct results by
+filtering out sequential duplicates.
 
 =cut
 
-package Attean::Plan::Slice 0.004 {
+package Attean::Plan::Unique 0.006 {
+	use Moo;
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	sub plan_as_string { return 'Unique' }
+	
+	sub impl {
+		my $self	= shift;
+		my $model	= shift;
+		my ($impl)	= map { $_->impl($model) } @{ $self->children };
+		return sub {
+			my $iter	= $impl->();
+			my $last	= '';
+			return $iter->grep(sub {
+				my $r	= shift;
+				my $s	= $r->as_string;
+				my $ok	= $s ne $last;
+				$last	= $s;
+				return $ok;
+			});
+		};
+	}
+}
+
+=item * L<Attean::Plan::Slice>
+
+Evaluates a sub-plan, and returns the results after optionally skipping some
+number of results ("offset") and limiting the total number of returned results
+("limit").
+
+=cut
+
+package Attean::Plan::Slice 0.006 {
 	use Moo;
 	use Types::Standard qw(Int);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
@@ -387,9 +506,12 @@ package Attean::Plan::Slice 0.004 {
 
 =item * L<Attean::Plan::Project>
 
+Evaluates a sub-plan and returns projected results by only keeping a fixed-set
+of variable bindings in each result.
+
 =cut
 
-package Attean::Plan::Project 0.004 {
+package Attean::Plan::Project 0.006 {
 	use Moo;
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
 	use Types::Standard qw(ArrayRef ConsumerOf);
@@ -418,9 +540,12 @@ package Attean::Plan::Project 0.004 {
 
 =item * L<Attean::Plan::OrderBy>
 
+Evaluates a sub-plan and returns the results after fully materializing and
+sorting is applied.
+
 =cut
 
-package Attean::Plan::OrderBy 0.004 {
+package Attean::Plan::OrderBy 0.006 {
 	use Moo;
 	use Types::Standard qw(HashRef ArrayRef InstanceOf Bool Str);
 	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
@@ -466,9 +591,11 @@ package Attean::Plan::OrderBy 0.004 {
 
 =item * L<Attean::Plan::Service>
 
+Evaluates a SPARQL query against a remove endpoint.
+
 =cut
 
-package Attean::Plan::Service 0.004 {
+package Attean::Plan::Service 0.006 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf Bool Str);
 	sub plan_as_string {
@@ -483,13 +610,20 @@ package Attean::Plan::Service 0.004 {
 	has 'sparql' => (is => 'ro', isa => Str, required => 1);
 	
 	sub tree_attributes { return qw(endpoint) };
+	sub impl {
+		my $self	= shift;
+		my $model	= shift;
+		die __PACKAGE__ . " unimplemented";
+	}
 }
 
 =item * L<Attean::Plan::Table>
 
+Returns a constant set of results.
+
 =cut
 
-package Attean::Plan::Table 0.004 {
+package Attean::Plan::Table 0.006 {
 	use Moo;
 	use Types::Standard qw(ArrayRef ConsumerOf);
 	use namespace::clean;
@@ -512,11 +646,40 @@ package Attean::Plan::Table 0.004 {
 	}
 }
 
+=item * L<Attean::Plan::Exists>
+
+Returns an iterator containing a single boolean term indicating whether any
+results were produced by evaluating the sub-plan.
+
+=cut
+
+package Attean::Plan::Exists 0.006 {
+	use Moo;
+	use Types::Standard qw(ArrayRef ConsumerOf);
+	use namespace::clean;
+	has variables => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Variable']]);
+	has rows => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::Result']]);
+	with 'Attean::API::Plan', 'Attean::API::UnaryQueryTree';
+	sub tree_attributes { return qw(variables rows) };
+	sub plan_as_string { return 'Exists' }
+	
+	sub impl {
+		my $self	= shift;
+		my $model	= shift;
+		my ($impl)	= map { $_->impl($model) } @{ $self->children };
+		return sub {
+			my $iter	= $impl->();
+			my $term	= ($iter->next) ? Attean::Literal->true : Attean::Literal->false;
+			return Attean::ListIterator->new(values => [$term], item_type => 'Attean::API::Term');
+		}
+	}
+}
+
 # =item * L<Attean::Algebra::Ask>
 # 
 # =cut
 # 
-# package Attean::Algebra::Ask 0.004 {
+# package Attean::Algebra::Ask 0.006 {
 # 	use Moo;
 # 	sub in_scope_variables { return; }
 # 	with 'Attean::API::Algebra', 'Attean::API::UnaryQueryTree';
@@ -538,7 +701,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Construct 0.004 {
+# package Attean::Algebra::Construct 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	has 'triples' => (is => 'ro', isa => ArrayRef[ConsumerOf['Attean::API::TriplePattern']]);
@@ -565,7 +728,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::Path 0.004 {
+# package Attean::Algebra::Path 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	sub in_scope_variables {
@@ -603,7 +766,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::NegatedPropertySet 0.004 {
+# package Attean::Algebra::NegatedPropertySet 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ArrayRef ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -624,7 +787,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::PredicatePath 0.004 {
+# package Attean::Algebra::PredicatePath 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::PropertyPath';
@@ -648,7 +811,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::InversePath 0.004 {
+# package Attean::Algebra::InversePath 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -664,7 +827,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::SequencePath 0.004 {
+# package Attean::Algebra::SequencePath 0.006 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "/" }
@@ -679,7 +842,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::AlternativePath 0.004 {
+# package Attean::Algebra::AlternativePath 0.006 {
 # 	use Moo;
 # 	with 'Attean::API::NaryPropertyPath';
 # 	sub separator { return "|" }
@@ -694,7 +857,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrMorePath 0.004 {
+# package Attean::Algebra::ZeroOrMorePath 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -710,7 +873,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::OneOrMorePath 0.004 {
+# package Attean::Algebra::OneOrMorePath 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
@@ -726,7 +889,7 @@ package Attean::Plan::Table 0.004 {
 # 
 # =cut
 # 
-# package Attean::Algebra::ZeroOrOnePath 0.004 {
+# package Attean::Algebra::ZeroOrOnePath 0.006 {
 # 	use Moo;
 # 	use Types::Standard qw(ConsumerOf);
 # 	with 'Attean::API::UnaryPropertyPath';
