@@ -7,14 +7,14 @@ Attean::Result - SPARQL Result
 
 =head1 VERSION
 
-This document describes Attean::Result version 0.005
+This document describes Attean::Result version 0.006
 
 =head1 SYNOPSIS
 
   use v5.14;
   use Attean;
-  my $result = Attean::Result->new( { name => $literal, homepage => $iri } );
-  my @vars = $result->variables; # ('name', 'iri')
+  my $result = Attean::Result->new(bindings => { name => $literal, homepage => $iri } );
+  my @vars = $result->variables; # ('name', 'homepage')
   my $term = $result->value('name'); # $term == $literal
 
 =head1 DESCRIPTION
@@ -29,7 +29,7 @@ It conforms to the L<Attean::API::Result|Attean::API::Binding> role.
 
 =cut
 
-package Attean::Result 0.005 {
+package Attean::Result 0.006 {
 	use Moo;
 	use Types::Standard qw(HashRef ConsumerOf);
 	use Attean::API::Binding;
